import { SemanticMark } from "./semantic-mark";

import { GrammarMarkType, Factory } from "@visactor/vgrammar-core";

import { PlotMakType } from "./enums";

import { field as getFieldAccessor } from "@visactor/vgrammar-util";

export class TreeSemanticMark extends SemanticMark {
    constructor(id) {
        super(PlotMakType.tree, id), Factory.getTransform(PlotMakType.tree) || this._logger.error("Please add this line of code: import { registerTreeTransforms } from 'vgrammar-hierarchy', \n        and run registerTreeTransforms() before use tree"), 
        Factory.getGlyph("treePath") || this._logger.error("Please add this line of code: import { registerTreePathGlyph } from '@visactor/vgrammar-core';\n        and run registerTreePathGlyph() before use tree");
    }
    setMarkType() {
        return GrammarMarkType.symbol;
    }
    setDefaultDataTransform() {
        var _a;
        return [ {
            type: "tree",
            width: {
                signal: "viewWidth"
            },
            height: {
                signal: "viewHeight"
            },
            nodeKey: null === (_a = this.spec.encode) || void 0 === _a ? void 0 : _a.node,
            flatten: !0
        } ];
    }
    getDataIdOfLink() {
        var _a, _b;
        return `${null !== (_b = null === (_a = this.spec.data) || void 0 === _a ? void 0 : _a.id) && void 0 !== _b ? _b : this.spec.id}-data-link`;
    }
    setMultipleData() {
        return [ {
            id: this.getDataIdOfFiltered(),
            transform: [ {
                type: "map",
                all: !0,
                callback: datum => datum[0].nodes
            } ]
        }, {
            source: this.getDataIdOfMain(),
            id: this.getDataIdOfLink(),
            transform: [ {
                type: "map",
                all: !0,
                callback: datum => datum[0].links
            } ]
        } ];
    }
    parseScaleByEncode(channel, option) {
        return "color" === channel ? {
            type: "ordinal",
            id: this.getScaleId("color"),
            domain: {
                data: this.getDataIdOfFiltered(),
                field: option
            },
            range: this.getPalette()
        } : null;
    }
    convertMarkEncode(encode) {
        var _a, _b, _c;
        const markEncoder = this.convertSimpleMarkEncode(encode), res = {
            x: {
                field: "x"
            },
            y: {
                field: "y"
            }
        };
        if (markEncoder.stroke && (res.stroke = markEncoder.stroke), markEncoder.color) {
            const scaleColorId = this.getScaleId("color"), colorAccessor = getFieldAccessor(markEncoder.color.field);
            res.fill = (datum, el, params) => {
                const scale = params[scaleColorId];
                return (null == datum ? void 0 : datum.datum) ? scale.scale(colorAccessor(datum.datum[datum.datum.length - 1])) : void 0;
            };
        } else res.fill = null !== (_b = null === (_a = this.spec.style) || void 0 === _a ? void 0 : _a.fill) && void 0 !== _b ? _b : null === (_c = this.getPalette()) || void 0 === _c ? void 0 : _c[0];
        return res;
    }
    setLabelTextGetter(channel, option) {
        const textGetter = getFieldAccessor(channel);
        return (datum, el, params) => textGetter(datum.datum[datum.datum.length - 1]);
    }
    setMainMarkSpec() {
        return {
            key: "key"
        };
    }
    setMultiMarksSpec() {
        if (!this.spec.label) return null;
        const marks = [];
        return marks.push({
            type: "glyph",
            glyphType: "treePath",
            from: {
                data: this.getDataIdOfLink()
            },
            key: "key",
            zIndex: -1,
            encode: {
                update: {
                    x0: {
                        field: "x0"
                    },
                    x1: {
                        field: "x1"
                    },
                    y0: {
                        field: "y0"
                    },
                    y1: {
                        field: "y1"
                    },
                    thickness: 1,
                    round: !0,
                    stroke: "#333"
                }
            }
        }), marks;
    }
}

TreeSemanticMark.type = PlotMakType.tree;
//# sourceMappingURL=tree.js.map